import josx.platform.rcx.*;

public class Balance extends Thread implements SensorListener {
	private final int BALANCE_TIME_TOLERANCE = 120;
	private CircuitRobot circuitRobot;
	private boolean forwardA, forwardC;
	private boolean stop, stopHandlingActive;
	public boolean isHorizontal;
	
	public Balance(CircuitRobot cr) {
		circuitRobot = cr;
	}
	
	// run-Methode des Threads
	public void run() {
		Sensor.S2.setTypeAndMode(
				SensorConstants.SENSOR_TYPE_TOUCH,
				SensorConstants.SENSOR_MODE_BOOL);
		Sensor.S2.addSensorListener(this);
		
		isHorizontal = !getTouchValue();
		stop = true;

		if (stopHandlingActive) {
			while (circuitRobot.isAlive()) {
				if (!isHorizontal) {
					Motor.A.stop();
					Motor.C.stop();
					pause(50);
				
					if (!stop) {
						if (forwardA) Motor.A.forward(); else Motor.A.backward();
						if (forwardC) Motor.C.forward(); else Motor.C.backward();
					}
					pause(50);
				}
			}
		}
	}
	
	// Stop-Modus ein- bzw. ausschalten
	public void setStopHandling() {
		TextLCD.print("STOP");
		TextLCD.printChar(stopHandlingActive ? 'Y' : 'N', 0);
		LCD.refresh();

		while (!Button.PRGM.isPressed()) {
			if (Button.VIEW.isPressed()) {
				stopHandlingActive = !stopHandlingActive;
				TextLCD.printChar(stopHandlingActive ? 'Y' : 'N', 0);
				LCD.refresh();
				pause(200);
			}
		}
		while (Button.PRGM.isPressed());
		
		LCD.clear();
		LCD.refresh();
		Sound.twoBeeps();
	}

	

	// Motorenvariablen auf Linksdrehung setzen 
	public void left() {
		forwardA = false;
		forwardC = true;
		stop = false;
	}
	
	// Motorenvariablen auf Rechtsdrehung setzen 
	public void right() {
		forwardA = true;
		forwardC = false;
		stop = false;
	}
	
	// Motorenvariablen auf Vorwaertsbewegung setzen 
	public void forward() {
		forwardA = true;
		forwardC = true;
		stop = false;
	}
	
	// Motorenvariablen auf Stillstand setzen 
	public void stop() {
		stop = true;
	}
	
	// boolean-Wert des Tastsensors zurueckgeben
	private boolean getTouchValue() {
		return Sensor.S2.readBooleanValue();
	}
	
	// Roboterneigung auf dem Display anzeigen
	public void showPosition() {
		TextLCD.printChar(isHorizontal ? '-' : '/', 0);
		LCD.refresh();
	}
	
	// Zustand eines Sensorwertes ueberwachen
	public void stateChanged(Sensor aSource, int aOldValue, int aNewValue) {
		// Horizontal-Wert (isHorizontal) nur aendern, wenn der negierte
		// Tastsensor-Wert und Horizontal-Wert verschieden sind
		// (also wenn beide Werte gleich sind)
		if (getTouchValue() == isHorizontal) {
			// Verzoegerung, um eventuell auftretende kurze Kontakte
			// des Tastsensors bei Richtungswechsel abzufangen
			pause(BALANCE_TIME_TOLERANCE);

			boolean isTouching = getTouchValue();
			if (isTouching == isHorizontal) {
				// Roboter ist waagerecht, wenn
				// Tastsensor nicht gedrueckt ist
				isHorizontal = !isTouching;

				// Neigung des Roboter ausgeben
				showPosition();
				Sound.beep();
			}
		}
	}
	
	// Zeitverzoegerung (in ms)
	public void pause(int time) {
		try {
			sleep(time);
		} catch (Exception exception) {}
	}
}
